//////////////////////////////////
/// audio interface to DE1-SoC ///
/// March 2016                 ///
/// part 2 introducing serial  ///
/// serial shift registers     ///
/// both in and out            ///
//////////////////////////////////

module audio (
    address,
    chipselect,
    readdata,
    read,
    write,
    writedata,
    aud_xclk	, // clock 12. MHz?
    bclk 		, // bit stream clock
    adclrck	, // left right clock ADC
    adcdat	, // data stream ADC
    daclrck	, // left right clock DAC
    dacdat	, // data stream DAC
    sclk		, // serial clock I2C
    sdat		, // serail data I2C
    swt		,
    clk		,
    reset,
    gpio,
    led_op,
    sw
);

input [15:0] address;
output reg [15:0] readdata;
input [15:0] writedata;
input chipselect;
input write;
input read;

input adcdat;
input swt;
input clk;
input bclk;

input reset;

input adclrck;
input daclrck;
inout sdat;

output aud_xclk;
output sclk;
output dacdat;
output[7:0] gpio;
output [9:0] led_op;
input [9:0] sw;


// THIS DOESN'T AFFECT ANYTHING
parameter n_samples = 2048;
// LEFT FOR QUARTUS, DON'T DELETE


I2C_programmer u1(
    .RESET(swt),			//  clock enable 
    .i2c_clk(clk),			//  50 Mhz clk from DE1-SoC
    .I2C_SCLK(sclk),		// I2C clock 40K
    .TRN_END(TRN_END),
    .ACK(ACK),
    .ACK_enable(ACK_enable),
    .I2C_SDATA(sdat)		// bi directional serial data 
);

//serial_adc u2 (
//    .serial_adc(adcdat),		// 32 bit serial in data
//    .SADCL(serial_lf),
//    .SADCR(serial_rt), 
//    .adc_lr(adclrck),
//    .clk(clk),				// 50 KHz clock
//    .enable(swt)			// master reset
//);

serial_dac u3(
    .serial_dac(dacdat),		// 32 bit serial in data
    .SDACL(hps_op),
    .SDACR(hps_op), 
    .dac_lr(daclrck),
    .clk(bclk),				// 50 KHz clock
    .enable(swt)			// master reset
);

hpsfreq h1(
    .clk(clk),
    .chipselect(chipselect),
    .write(write),
    .addr(address),
    .writedata(writedata),
    .readdata(readout),
    .read(read),
    .wt_op1(sine_op1),
    .wt_addr1(ad1),
    .wt_op2(sine_op2),
    .wt_addr2(ad2),
    .wt_op3(sine_op3),
    .wt_addr3(ad3)
);

key_parser k1(
    .clk(clk),
    .chipselect(chipselect),
    .write(write),
    .address(address),
    .writedata(writedata),
    .kp_1(kp1_out),
    .kp_2(kp2_out),
    .kp_3(kp3_out)
);

summer s1(
    .ip1(sine_op1),
    .ip2(sine_op2),
    .ip3(sine_op3),
    .summed_op(summed_op),
    .key_1(kp1_out_temp),
    .key_2(kp2_out_temp),
    .key_3(kp3_out_temp)
);

asr_envelope asr1(
    .clk(clk),
    .chipselect(chipselect),
    .write(write),
    .address(address),
    .writedata(writedata),
    .sample_in(summed_op),
    .pressed(is_key_pressed),
    .sample_out(sample_out)
);

wire is_key_pressed ;

assign is_key_pressed = (kp1_out > 0 || kp2_out > 0 || kp3_out > 0) ? 1'b1 : 1'b0;

// Unused but left in place
parameter S1 = 0;
parameter S2 = 0;

wire [15:0] readout;

wire [4:0] kp1_out;
wire [4:0] kp2_out;
wire [4:0] kp3_out;

// Data being written to DAC
// It expects 33bit
wire [32:0] hps_op;

wire [15:0] summed_op;
wire [15:0] sample_out;

// AD and op_addrs
wire [15:0] ad1 ;
assign ad1 = op_addr1[31:16];
reg [31:0] op_addr1 = 0; 

wire [15:0] ad2 ;
assign ad2 = op_addr2[31:16];
reg [31:0] op_addr2 = 0; 

wire [15:0] ad3 ;
assign ad3 = op_addr3[31:16];
reg [31:0] op_addr3 = 0; 
// AD and op_addrs

wire [15:0] sine_op1;
wire [15:0] sine_op2;
wire [15:0] sine_op3;


// op from hpsfreq
assign hps_op = {1'b0, sample_out, {16'b0}};
// op from hpsfreq

wire [4:0] sel;
assign sel = {sw[0], sw[1], sw[2], sw[3], sw[4]};

always @(posedge clk)
    readdata <= readout;


// remember last keypress to support env Release
reg [4:0] prev_kp1_out;
reg [4:0] prev_kp2_out;
reg [4:0] prev_kp3_out;
reg [4:0] kp1_out_temp;
reg [4:0] kp2_out_temp;
reg [4:0] kp3_out_temp;
// remember last keypress to support env Release
always @(posedge clk) begin
    if (is_key_pressed) begin
        kp1_out_temp <= kp1_out;
        prev_kp1_out <= kp1_out;
        kp2_out_temp <= kp2_out;
        prev_kp2_out <= kp2_out;
        kp3_out_temp <= kp3_out;
        prev_kp3_out <= kp3_out;
    end
    else begin
        kp1_out_temp <= prev_kp1_out;
        kp2_out_temp <= prev_kp2_out;
        kp3_out_temp <= prev_kp3_out;
    end
end


reg [31:0] incr1 = 0;
reg [31:0] incr2 = 0;
reg [31:0] incr3 = 0;

// voodoo voodoo
always @(*) begin 
    case(kp1_out_temp)
        5'd0  : incr1   = 32'b0;
        5'd1  : incr1   = 32'b0000_0000_0000_0001_0000_0000_0000_0000;
        5'd2  : incr1   = 32'b0000_0000_0000_0001_0000_1111_0011_1000; 
        5'd3  : incr1   = 32'b0000_0000_0000_0001_0001_1111_0101_1001; 
        5'd4  : incr1   = 32'b0000_0000_0000_0001_0011_0000_0110_1111; 
        5'd5  : incr1   = 32'b0000_0000_0000_0001_0100_0010_1000_1010; 
        5'd6  : incr1   = 32'b0000_0000_0000_0001_0101_0101_1011_1000; 
        5'd7  : incr1   = 32'b0000_0000_0000_0001_0110_1010_0000_1001;
        5'd8  : incr1   = 32'b0000_0000_0000_0001_0111_1111_1001_0001; 
        5'd9  : incr1   = 32'b0000_0000_0000_0001_1001_0110_0101_1111; 
        5'd10 : incr1   = 32'b0000_0000_0000_0001_1010_1110_1000_1001;
        5'd11 : incr1   = 32'b0000_0000_0000_0001_1100_1000_0010_0011; 
        5'd12 : incr1   = 32'b0000_0000_0000_0001_1110_0011_0100_0011; 
        5'd13 : incr1   = 32'b0000_0000_0000_0010_0000_0000_0000_0000;
        5'd14 : incr1   = 32'b0000_0000_0000_0010_0001_1110_0111_0001; 
        5'd15 : incr1   = 32'b0000_0000_0000_0010_0011_1110_1011_0011;
        5'd16 : incr1   = 32'b0000_0000_0000_0010_0110_0000_1101_1111;
        5'd17 : incr1   = 32'b0000_0000_0000_0010_1000_0101_0001_0100;
        5'd18 : incr1   = 32'b0000_0000_0000_0010_1010_1011_0111_0000; 
        5'd19 : incr1   = 32'b0000_0000_0000_0010_1101_0100_0001_0011;
        5'd20 : incr1   = 32'b0000_0000_0000_0010_1111_1111_0010_0010; 
        5'd21 : incr1   = 32'b0000_0000_0000_0011_0010_1100_1011_1111; 
        5'd22 : incr1   = 32'b0000_0000_0000_0011_0101_1101_0001_0011;
        5'd23 : incr1   = 32'b0000_0000_0000_0011_1001_0000_0100_0111;
        5'd24 : incr1   = 32'b0000_0000_0000_0011_1100_0110_1000_0110;
        5'd25 : incr1   = 32'b0000_0000_0000_0100_0000_0000_0000_0000;
        5'd26 : incr1   = 32'b0000_0000_0000_0100_0011_1100_1110_0011;
        5'd27 : incr1   = 32'b0000_0000_0000_0100_0111_1101_0110_0110;
        5'd28 : incr1   = 32'b0000_0000_0000_0100_1100_0001_1011_1111;
        5'd29 : incr1   = 32'b0000_0000_0000_0101_0000_1010_0010_1000; 
        5'd30 : incr1   = 32'b0000_0000_0000_0101_0101_0110_1110_0000;
        5'd31 : incr1   = 32'b0000_0000_0000_0101_1010_1000_0010_0111;

      default: 
           incr1 = 32'b0;
    endcase
end
// voodoo voodoo

// voodoo voodoo
always @(*) begin 
    case(kp2_out_temp)
        5'd0  : incr2   = 32'b0;
        5'd1  : incr2   = 32'b0000_0000_0000_0001_0000_0000_0000_0000;
        5'd2  : incr2   = 32'b0000_0000_0000_0001_0000_1111_0011_1000; 
        5'd3  : incr2   = 32'b0000_0000_0000_0001_0001_1111_0101_1001; 
        5'd4  : incr2   = 32'b0000_0000_0000_0001_0011_0000_0110_1111; 
        5'd5  : incr2   = 32'b0000_0000_0000_0001_0100_0010_1000_1010; 
        5'd6  : incr2   = 32'b0000_0000_0000_0001_0101_0101_1011_1000; 
        5'd7  : incr2   = 32'b0000_0000_0000_0001_0110_1010_0000_1001;
        5'd8  : incr2   = 32'b0000_0000_0000_0001_0111_1111_1001_0001; 
        5'd9  : incr2   = 32'b0000_0000_0000_0001_1001_0110_0101_1111; 
        5'd10 : incr2   = 32'b0000_0000_0000_0001_1010_1110_1000_1001;
        5'd11 : incr2   = 32'b0000_0000_0000_0001_1100_1000_0010_0011; 
        5'd12 : incr2   = 32'b0000_0000_0000_0001_1110_0011_0100_0011; 
        5'd13 : incr2   = 32'b0000_0000_0000_0010_0000_0000_0000_0000;
        5'd14 : incr2   = 32'b0000_0000_0000_0010_0001_1110_0111_0001; 
        5'd15 : incr2   = 32'b0000_0000_0000_0010_0011_1110_1011_0011;
        5'd16 : incr2   = 32'b0000_0000_0000_0010_0110_0000_1101_1111;
        5'd17 : incr2   = 32'b0000_0000_0000_0010_1000_0101_0001_0100;
        5'd18 : incr2   = 32'b0000_0000_0000_0010_1010_1011_0111_0000; 
        5'd19 : incr2   = 32'b0000_0000_0000_0010_1101_0100_0001_0011;
        5'd20 : incr2   = 32'b0000_0000_0000_0010_1111_1111_0010_0010; 
        5'd21 : incr2   = 32'b0000_0000_0000_0011_0010_1100_1011_1111; 
        5'd22 : incr2   = 32'b0000_0000_0000_0011_0101_1101_0001_0011;
        5'd23 : incr2   = 32'b0000_0000_0000_0011_1001_0000_0100_0111;
        5'd24 : incr2   = 32'b0000_0000_0000_0011_1100_0110_1000_0110;
        5'd25 : incr2   = 32'b0000_0000_0000_0100_0000_0000_0000_0000;
        5'd26 : incr2   = 32'b0000_0000_0000_0100_0011_1100_1110_0011;
        5'd27 : incr2   = 32'b0000_0000_0000_0100_0111_1101_0110_0110;
        5'd28 : incr2   = 32'b0000_0000_0000_0100_1100_0001_1011_1111;
        5'd29 : incr2   = 32'b0000_0000_0000_0101_0000_1010_0010_1000; 
        5'd30 : incr2   = 32'b0000_0000_0000_0101_0101_0110_1110_0000;
        5'd31 : incr2   = 32'b0000_0000_0000_0101_1010_1000_0010_0111;

      default: 
           incr2 = 32'b0;
    endcase
end
// voodoo voodoo

// voodoo voodoo
always @(*) begin 
    case(kp3_out_temp)
        5'd0  : incr3   = 32'b0;
        5'd1  : incr3   = 32'b0000_0000_0000_0001_0000_0000_0000_0000;
        5'd2  : incr3   = 32'b0000_0000_0000_0001_0000_1111_0011_1000; 
        5'd3  : incr3   = 32'b0000_0000_0000_0001_0001_1111_0101_1001; 
        5'd4  : incr3   = 32'b0000_0000_0000_0001_0011_0000_0110_1111; 
        5'd5  : incr3   = 32'b0000_0000_0000_0001_0100_0010_1000_1010; 
        5'd6  : incr3   = 32'b0000_0000_0000_0001_0101_0101_1011_1000; 
        5'd7  : incr3   = 32'b0000_0000_0000_0001_0110_1010_0000_1001;
        5'd8  : incr3   = 32'b0000_0000_0000_0001_0111_1111_1001_0001; 
        5'd9  : incr3   = 32'b0000_0000_0000_0001_1001_0110_0101_1111; 
        5'd10 : incr3   = 32'b0000_0000_0000_0001_1010_1110_1000_1001;
        5'd11 : incr3   = 32'b0000_0000_0000_0001_1100_1000_0010_0011; 
        5'd12 : incr3   = 32'b0000_0000_0000_0001_1110_0011_0100_0011; 
        5'd13 : incr3   = 32'b0000_0000_0000_0010_0000_0000_0000_0000;
        5'd14 : incr3   = 32'b0000_0000_0000_0010_0001_1110_0111_0001; 
        5'd15 : incr3   = 32'b0000_0000_0000_0010_0011_1110_1011_0011;
        5'd16 : incr3   = 32'b0000_0000_0000_0010_0110_0000_1101_1111;
        5'd17 : incr3   = 32'b0000_0000_0000_0010_1000_0101_0001_0100;
        5'd18 : incr3   = 32'b0000_0000_0000_0010_1010_1011_0111_0000; 
        5'd19 : incr3   = 32'b0000_0000_0000_0010_1101_0100_0001_0011;
        5'd20 : incr3   = 32'b0000_0000_0000_0010_1111_1111_0010_0010; 
        5'd21 : incr3   = 32'b0000_0000_0000_0011_0010_1100_1011_1111; 
        5'd22 : incr3   = 32'b0000_0000_0000_0011_0101_1101_0001_0011;
        5'd23 : incr3   = 32'b0000_0000_0000_0011_1001_0000_0100_0111;
        5'd24 : incr3   = 32'b0000_0000_0000_0011_1100_0110_1000_0110;
        5'd25 : incr3   = 32'b0000_0000_0000_0100_0000_0000_0000_0000;
        5'd26 : incr3   = 32'b0000_0000_0000_0100_0011_1100_1110_0011;
        5'd27 : incr3   = 32'b0000_0000_0000_0100_0111_1101_0110_0110;
        5'd28 : incr3   = 32'b0000_0000_0000_0100_1100_0001_1011_1111;
        5'd29 : incr3   = 32'b0000_0000_0000_0101_0000_1010_0010_1000; 
        5'd30 : incr3   = 32'b0000_0000_0000_0101_0101_0110_1110_0000;
        5'd31 : incr3   = 32'b0000_0000_0000_0101_1010_1000_0010_0111;

      default: 
           incr3 = 32'b0;
    endcase
end
// voodoo voodoo


// access 3 memories
always @(negedge daclrck)  begin
    if(!write) begin 
        if (!sw[0]) begin
            if (op_addr1[31:16] >= 32767) op_addr1 <= 0;
            else op_addr1 <= op_addr1 + incr1; 
        end
        else begin
            if (op_addr1[31:16] == 0) op_addr1[31:16] <= 32767;
            else op_addr1 <= op_addr1 - incr1; 
        end
    end
end

always @(negedge daclrck)  begin
    if(!write) begin 
        if (!sw[0]) begin
            if (op_addr2[31:16] >= 32767) op_addr2 <= 0;
            else op_addr2 <= op_addr2 + incr2;  
        end
        else begin
            if (op_addr2[31:16] == 0) op_addr2[31:16] <= 32767;
            else op_addr2 <= op_addr2 - incr2;  
        end
    end
end

always @(negedge daclrck)  begin
    if(!write) begin 
        if (!sw[0]) begin
            if (op_addr3[31:16] >= 32767) op_addr3 <= 0;
            else op_addr3 <= op_addr3 + incr3;  
        end
        else begin
            if (op_addr3[31:16] == 0) op_addr3[31:16] <= 32767;
            else op_addr3 <= op_addr3 - incr3;
        end
    end
end



  ///////////////////////////////////////////////////
  /// variables and parameter for state machines  ///
  ///////////////////////////////////////////////////


  parameter clk_freq = 50000000;  // 50 Mhz
  parameter i2c_freq = 12288000;  // 12.288 Mhz

  wire[32:0] serial_lf;
  wire[32:0] serial_rt;

  ///////////////////////////////////////////////////////
  ////// I2C clock (50 Mhz)used for DE1-SoC video in chip ///
  ///////////////////////////////////////////////////////
 reg clk_by2 = 0;
 reg clk_by4 = 0;

always @(posedge clk)
    clk_by2 = ~clk_by2;

always @(posedge clk_by2)
    clk_by4 = ~clk_by4;

        wire sclk; 
        wire sdat; 

        /////////////////////////////////////////
        //// internal signals
        ///////////////////////////////////////// 
        wire ACK ;
        wire ACK_enable;
        wire [23:0] data_23;
        wire TRN_END;
        reg ctrl_clk;
        reg [15:0] clk_div;  // clock divider

        assign aud_xclk = clk_by4;// ctrl_clk; 

        assign gpio[0] = clk_by4;//ctrl_clk;
        assign gpio[1] = bclk;
        assign gpio[2] = dacdat;
        assign gpio[3] = daclrck;
        assign gpio[4] = adcdat;
        assign gpio[5] = adclrck;
        assign gpio[6] = chipselect;
        assign gpio[7] = write;
        //assign readdata[7:0] = serial_lf[14:7];

endmodule
